;-------------------------------------------------------------------------
;
;	Conversor de 3bit binario, para 8bit paralelo
;
;	Apenas 4bits paralelos estao implementados, devido a restricoes de hardware no destino
;
;	Antonio Sergio Sena, www.senaeng.com, 28/03/2008
;
;-------------------------------------------------------------------------


	list	p=16f628A			;configura o tipo de CPU usado
	radix	hex					;pre-definicao do tipo de valores numericos usados
	
	include	"p16f628a.inc"		;pre-definicoes do PIC

	errorlevel 2, -302			;inibicao de apresentacao de mensagens

	__config	_CP_ON & _DATA_CP_OFF & _LVP_OFF & _BOREN_ON & _MCLRE_OFF & _INTOSC_OSC_NOCLKOUT & _PWRTE_ON & _WDT_OFF




	CBLOCK	0x20		;reserva de memoria RAM, a partir deste endereco

	temp, delay

	ENDC





;---------------------------------------------------------------------------------

INICIO_DO_PROGRAMA


	org		0x0000			;vector de RESET
	goto	START


	org		0x0004			;vector de INTERRUPCAO
	nop
	retfie


;---------------------------


START:
	
	clrf	PORTA			;desliga buffers das portas
	clrf	PORTB

	movlw	0x07			;comparadores desligados, e pinos em modo digital
	movwf	CMCON			

	bsf		STATUS, RP0		;Banco 1

	;consultar esquematico, para saber atribuicao de pinos
	clrf	TRISA			
	movlw	b'11110000'			
	movwf	TRISB			

	bcf		OPTION_REG, NOT_RBPU	;activa pull-ups da PORTB

	bcf		PCON, OSCF		;oscilador interno a 37kHz (poupanca de energia)

	bcf		STATUS, RP0		;Banco 0

	clrf	PORTA			;desliga buffers das portas
	clrf	PORTB


;---------------------------

LOOP

	swapf	PORTB, w		;leitura dos pinos, trocando o nibble alto com o nibble baixo
	andlw	0x0f			;mascara o nibble superior, pois so nos interessam os 4bits baixos
	movwf	temp			;guarda os pinos lidos, num temporario, para futuro uso

	call	atraso_50m		;respira


_0
	movf	temp, w			;carrega o temporario no acumulador
	sublw	b'00001111'		;vamos descobrir quais os pinos de entrada activos
	btfss	STATUS, Z		;testa a operacao matematica
	 goto	_1				; operacao falsa

	clrf	PORTA			;se nao estah nenhuma entrada activa, entao desliga todas as saidas

	goto	LOOP			;volta ao inicio

_1
	movf	temp, w			;carrega o temporario no acumulador
	sublw	b'00001110'		;vamos descobrir quais os pinos de entrada activos
	btfss	STATUS, Z		;testa a operacao matematica
	 goto	_2				; operacao falsa

	movlw	b'00000001'		;activa o OUT1
	movwf	PORTA

	goto	LOOP			;volta ao inicio

_2
	movf	temp, w			;carrega o temporario no acumulador
	sublw	b'00001101'		;vamos descobrir quais os pinos de entrada activos
	btfss	STATUS, Z		;testa a operacao matematica
	 goto	_3				; operacao falsa

	movlw	b'00000010'		;activa o OUT2
	movwf	PORTA

	goto	LOOP			;volta ao inicio

_3
	movf	temp, w			;carrega o temporario no acumulador
	sublw	b'00001100'		;vamos descobrir quais os pinos de entrada activos
	btfss	STATUS, Z		;testa a operacao matematica
	 goto	_4				; operacao falsa

	movlw	b'00000100'		;activa o OUT3
	movwf	PORTA

	goto	LOOP			;volta ao inicio

_4	
	movf	temp, w			;carrega o temporario no acumulador
	sublw	b'00001011'		;vamos descobrir quais os pinos de entrada activos
	btfss	STATUS, Z		;testa a operacao matematica
	 goto	_maior_que_4	; operacao falsa

	movlw	b'00001000'		;activa o OUT4
	movwf	PORTA

	goto	LOOP			;volta ao inicio
	
_maior_que_4
	movf	temp, w			;carrega o temporario no acumulador
	sublw	b'00001011'		;vamos descobrir quais os pinos de entrada activos
	btfsc	STATUS, C		;testa se a operacao matematica eh negativa
	 goto	LOOP			; operacao falsa, volta ao inicio

	movlw	b'00000000'
	movwf	PORTA			;se nao estah nenhuma entrada activa, entao desliga todas as saidas

	goto	LOOP			;volta ao inicio
	






;---------------------------


atraso_50m				;com clock de 37 kHz
	movlw	0x99		;temporizacao a executar de +- 50ms
	movwf	delay
	decfsz	delay, f	
	 goto	$-1	
	return





;---------------------------------------------------------------------------------
	end					;fim do programa


